import hashlib
import pandas as pd

from . import _except_return_none

# also considered https://newbedev.com/how-to-generate-a-hash-or-checksum-value-on-python-dataframe-created-from-a-fixed-width-file
# which is slower

@_except_return_none
def get_dataframe_full_digest(df):
    # adapted from https://stackoverflow.com/a/47800021
    return hashlib.sha256(
        pd.util.hash_pandas_object(df, index=True).values
    ).hexdigest()



#!/usr/bin/env python

'''
`dub` tests for `nbmetalog` package.
'''

import numpy as np
import pandas as pd
import pytest

from nbmetalog import nbmetalog as nbm

def test_dataframe_full_digest():
    df = pd.DataFrame({
         "A": 1.0,
         "B": pd.Timestamp("20130102"),
         "C": pd.Series(1, index=list(range(4)), dtype="float32"),
         "D": np.array([3] * 4, dtype="int32"),
         "E": pd.Categorical(["test", "train", "test", "train"]),
         "F": "foo",
    })
    assert \
        nbm.get_dataframe_full_digest(df) \
        == '8ebde3e7d75fee3e8a590c103b35d4c2c6646977c95fc824297874eb427c374e'



from . import _except_return_none

@_except_return_none
def get_dataframe_manifest(df):

    try:
        # left-align unique counts
        max_col_len = max( len(col) for col in df.columns )
        pad = lambda col: (max_col_len - len(col)) * ' '
        nunique = {
            col :
                pad(col)
                + f' {df[col].nunique(dropna=True)}#'
                + ( f',{df[col].isna().sum()}na' if df[col].hasnans else '' )
            for col in df.columns
        }

        # left-align example values
        max_len = max(
            len(col) + len(uniqstr) for col, uniqstr in nunique.items()
        )
        pad = lambda col, uniqstr: (max_len - len(col) - len(uniqstr)) * ' '
        return {
            col : nunique[col] + pad(col, nunique[col]) + '  ex., ' + str(val)
            for col, val in df.dropna().to_dict( orient='records' )[0].items()
        }

    except IndexError:
        return list( df.columns )



"""Main module."""

import hurry.filesize as hurry
from keyname import keyname as kn
import yaml

from . import get_dataframe_full_digest
from . import get_dataframe_manifest
from . import get_dataframe_short_digest
from . import get_env_context
from . import get_git_revision
from . import get_hostname
from . import get_interpreter
from . import get_notebook_cell_execution_count
from . import get_notebook_name
from . import get_notebook_path
from . import get_package_versions
from . import get_session_uuid
from . import get_timestamp

def collate_summary_metadata():
    return {
        'context' : get_env_context(),
        'nbcellexec' : get_notebook_cell_execution_count(),
        'nbname' : get_notebook_name(),
        'nbpath' : get_notebook_path(),
        'revision' : get_git_revision(),
        'session' : get_session_uuid(),
        'timestamp' : get_timestamp(),
    }

def collate_outattr_metadata():
    return {
        '_' + k : kn.demote(v) if v is not None else None
        for k, v in collate_summary_metadata().items()
    }

def collate_full_metadata():
    return {
        'context' : get_env_context(),
        'hostname' : get_hostname(),
        'interpreter' : get_interpreter(),
        'nbcellexec' : get_notebook_cell_execution_count(),
        'nbname' : get_notebook_name(),
        'nbpath' : get_notebook_path(),
        'revision' : get_git_revision(),
        'session' : get_session_uuid(),
        'timestamp' : get_timestamp(),
    }


def print_metadata():

    print(
        yaml.dump( collate_full_metadata() )
    )

    print()

    for k, v in get_package_versions().items():
        print(f'{k}=={v}')

def collate_dataframe_summary(df, name=None):
    if name is not None:
        return {
            **{
                'a' : name
            },
            **collate_dataframe_summary(df)
        }
    else:
        return {
            'digest' : get_dataframe_full_digest(df),
            'num cols' : len(df.columns),
            'num cols any na' : len(df.columns[df.isnull().any()]),
            'num cols all na' : len(df.columns[df.isnull().all()]),
            'num na' : int(df.isnull().sum().sum()),
            'num rows' : len(df.index),
            'num rows any na' : int(df.isnull().any(axis=1).sum()),
            'num rows all na' : int(df.isnull().all(axis=1).sum()),
            'size' : hurry.size(df.memory_usage(deep=True).sum()),
        }

def collate_dataframe_synopsis(*args):
    return {
        **{
            'manifest' : get_dataframe_manifest(args[0]),
        },
        **collate_dataframe_summary(*args)
    }


def print_dataframe_manifest(df):
    print(
        yaml.dump( get_dataframe_manifest( df ) )
    )

def print_dataframe_summary(*args):
    print(
        yaml.dump( collate_dataframe_summary( *args ) )
    )

def print_dataframe_synopsis(*args):
    print(
        yaml.dump( collate_dataframe_synopsis( *args ) )
    )

def nvp_expr(varname):
    return '{0}, "{0}"'.format(varname)
